import click

from .data import rewrite_data, get_tags, get_sessions, get_active_session


@click.command('tags')
@click.option('--new')
def tag(
    new: str
):

    mytags = get_tags()

    if new:
        if new in mytags:
            click.echo(f'tag {new} already exists')
            exit(1)
        else:
            mysessions = get_sessions()
            active_session = get_active_session()

            new_data = {
                'tags': [*mytags, new],
                'active_session': active_session,
                'sessions': [*mysessions]
            }

            rewrite_data(new_data)
            click.echo(f'new tag - {new}')
            exit(0)
    else:
        if mytags == []:
            click.echo('you do not have tags yet')
            click.echo('type "raw tags --new <NAME>" to add a new tag')
            exit(1)
        for tag in mytags:
            click.echo(f'* {tag}')



import json
from pathlib import Path


DATA_PATH = Path.home() / '.rawdata.json'


def read_data() -> dict:
    with open(DATA_PATH, 'r', encoding='utf-8') as file:
        data = json.load(file)

    return data


def get_field(data_dict: dict, field: str):
    value = data_dict.get(field)

    return value


def rewrite_data(new_data: dict) -> None:
    with open(DATA_PATH, 'w', encoding='utf-8') as file:
        json.dump(new_data, file, ensure_ascii=False, indent=4)

    return None


# custom methods

def get_tags() -> list:
    data = read_data()
    tags: list = get_field(data, 'tags')

    return tags if tags else []

def get_sessions() -> list[dict]:
    data = read_data()
    sessions: list = get_field(data, 'sessions')

    return sessions if sessions else []

def get_active_session() -> dict | None:
    data = read_data()
    active_session = get_field(data, 'active_session')

    return active_session



from datetime import datetime
import time

import click

from .data import rewrite_data, get_tags, get_active_session, get_sessions, read_data

@click.command('begin')
@click.argument('tags', nargs=-1)
def begin_session(tags: tuple):

    active_session = get_active_session()

    if active_session:
        click.echo('you already started a session')
        exit(1)

    if tags == ():
        click.echo('you should provide at least one tag to begin a session')
        exit(1)
    
    mytags = get_tags()
    sessions = get_sessions()

    for tag in tags:
        if tag not in mytags:
            click.echo(f'tag {tag} does not exist yet')
            exit(1)

    start_time = datetime.now()

    new_data = {
        'tags': [*mytags],
        'active_session': {
            'tags': [*tags],
            'start time': f'{start_time}',
            'breaks': 0
        },
        'sessions': [*sessions]
    }

    rewrite_data(new_data)

    click.echo('session started')

@click.command('finish')
def finish_session():

    active_session = get_active_session()

    if not active_session:
        click.echo('you did not start a session yet')
        exit(1)

    mytags = get_tags()
    mysessions = get_sessions()

    start_time = datetime.fromisoformat(active_session['start time'])
    end_time = datetime.now()
    breaks: int = active_session.get('breaks')
    timedelta = ((end_time - start_time).seconds) - breaks
    hours = timedelta // 3600
    timedelta -= hours*3600
    minutes = timedelta // 60
    timedelta -= minutes * 60
    seconds = timedelta

    total_time = {
        'hours': hours,
        'minutes': minutes,
        'seconds': seconds
    }

    new_session = {
        'tags': [*(active_session.get('tags'))],
        'start time': f'{start_time}',
        'end time': f'{end_time}',
        'breaks': breaks,
        'total time': total_time
    }

    new_data = {
        'tags': [*mytags],
        'active_session': {},
        'sessions': [*mysessions, new_session]
    }

    rewrite_data(new_data=new_data)

    click.echo('session finished')
    click.echo('')

@click.command('pause')
def pause_session():

    active_session = get_active_session()

    if not active_session:
        click.echo('you did not start a session yet')
        exit(1)

    tags: list = active_session.get('tags')
    breaks: int = active_session.get('breaks')
    data = read_data()
    
    while True:
        time.sleep(1)
        breaks += 1

        active_session['breaks'] = breaks
        data['active_session'] = active_session
        rewrite_data(data)



from importlib.metadata import version

import click

from .tags import tag
from .sessions import begin_session, finish_session, pause_session


@click.command('version')
def show_version():
    v = version('raww')

    click.echo(f'raw {v}')


@click.group()
def raw():
    ...


## commands ##

# std 
raw.add_command(show_version)

# tags
raw.add_command(tag)

# sessions
raw.add_command(begin_session, name='begin')
raw.add_command(begin_session, name='start')
raw.add_command(finish_session, name='finish')
raw.add_command(finish_session, name='stop')
raw.add_command(pause_session, name='pause')
